@@ -2,7 +2,7 @@
 #
 # Classes for the Hatari UI dialogs
 #
-# Copyright (C) 2008 by Eero Tamminen <eerot@sf.net>
+# Copyright (C) 2008-2012 by Eero Tamminen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,7 +23,8 @@ import pango
 
 from uihelpers import UInfo, HatariTextInsert, create_table_dialog, \
      table_add_entry_row, table_add_widget_row, table_add_separator, \
-     create_button, FselEntry
+     table_add_radio_rows, table_set_col_offset, create_button, FselEntry, \
+     FselAndEjectFactory
 
 
 # -----------------
@@ -47,13 +48,14 @@ subclasses overriding run() require also an argument."""
 # Note/Todo/Error/Ask dialogs
 
 class NoteDialog(HatariUIDialog):
+    button = gtk.BUTTONS_OK
     icontype = gtk.MESSAGE_INFO
     textpattern = "\n%s"
     def run(self, text):
         "run(text), show message dialog with given text"
         dialog = gtk.MessageDialog(self.parent,
         gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
-        self.icontype, gtk.BUTTONS_CLOSE, self.textpattern % text)
+        self.icontype, self.button, self.textpattern % text)
         dialog.run()
         dialog.destroy()
 
@@ -61,6 +63,7 @@ class TodoDialog(NoteDialog):
     textpattern = "\nTODO: %s"
 
 class ErrorDialog(NoteDialog):
+    button = gtk.BUTTONS_CLOSE
     icontype = gtk.MESSAGE_ERROR
     textpattern = "\nERROR: %s"
 
@@ -85,13 +88,13 @@ class AboutDialog(HatariUIDialog):
         dialog.set_transient_for(parent)
         dialog.set_name(UInfo.name)
         dialog.set_version(UInfo.version)
-        dialog.set_website("http://hatari.berlios.de/")
+        dialog.set_website("http://hatari.tuxfamily.org/")
         dialog.set_website_label("Hatari emulator www-site")
         dialog.set_authors(["Eero Tamminen"])
         dialog.set_artists(["The logo is from Hatari"])
         dialog.set_logo(gtk.gdk.pixbuf_new_from_file(UInfo.logo))
         dialog.set_translator_credits("translator-credits")
-        dialog.set_copyright("UI copyright (C) 2008-2009 by Eero Tamminen")
+        dialog.set_copyright(UInfo.copyright)
         dialog.set_license("""
 This software is licenced under GPL v2 or later.
 
@@ -109,28 +112,30 @@ class InputDialog(HatariUIDialog):
             gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
             ("Close", gtk.RESPONSE_CLOSE))
         
-        tips = gtk.Tooltips()
         entry = gtk.Entry()
         entry.connect("activate", self._entry_cb)
         insert = create_button("Insert", self._entry_cb)
-        tips.set_tip(insert, "Insert text to Hatari window")
+        insert.set_tooltip_text("Insert given text to Hatari window")
+        enter = create_button("Enter key", self._enter_cb)
+        enter.set_tooltip_text("Simulate Enter key press")
 
         hbox1 = gtk.HBox()
         hbox1.add(gtk.Label("Text:"))
         hbox1.add(entry)
         hbox1.add(insert)
+        hbox1.add(enter)
         dialog.vbox.add(hbox1)
 
-        rclick = gtk.Button("Rightclick")
-        tips.set_tip(rclick, "Simulate Atari left button double-click")
+        rclick = gtk.Button("Right click")
         rclick.connect("pressed", self._rightpress_cb)
         rclick.connect("released", self._rightrelease_cb)
-        dclick = create_button("Doubleclick", self._doubleclick_cb)
-        tips.set_tip(dclick, "Simulate Atari rigth button click")
+        rclick.set_tooltip_text("Simulate Atari right button press & release")
+        dclick = create_button("Double click", self._doubleclick_cb)
+        dclick.set_tooltip_text("Simulate Atari left button double-click")
 
         hbox2 = gtk.HBox()
-        hbox2.add(rclick)
         hbox2.add(dclick)
+        hbox2.add(rclick)
         dialog.vbox.add(hbox2)
 
         dialog.show_all()
@@ -143,14 +148,17 @@ class InputDialog(HatariUIDialog):
             HatariTextInsert(self.hatari, text)
             self.entry.set_text("")
 
+    def _enter_cb(self, widget):
+        self.hatari.insert_event("keypress 28") # Enter key scancode
+
     def _doubleclick_cb(self, widget):
         self.hatari.insert_event("doubleclick")
 
     def _rightpress_cb(self, widget):
-        self.hatari.insert_event("rightpress")
+        self.hatari.insert_event("rightdown")
 
     def _rightrelease_cb(self, widget):
-        self.hatari.insert_event("rightrelease")
+        self.hatari.insert_event("rightup")
 
     def run(self, hatari):
         "run(hatari), do text/mouse click input"
@@ -266,37 +274,39 @@ class ResetDialog(HatariUIDialog):
 # ----------------------------------
 # Floppy image dialog
 
-class DiskDialog(HatariUIDialog):
+class FloppyDialog(HatariUIDialog):
     def _create_dialog(self, config):
-        table, self.dialog = create_table_dialog(self.parent, "Floppy images", 9)
-        
+        table, self.dialog = create_table_dialog(self.parent, "Floppy images", 4, 2)
+        factory = FselAndEjectFactory()
+
         row = 0
         self.floppy = []
         path = config.get_floppydir()
         for drive in ("A", "B"):
             label = "Disk %c:" % drive
-            fsel = gtk.FileChooserButton(label)
-            # Hatari cannot access URIs
-            fsel.set_local_only(True)
-            fsel.set_width_chars(12)
-            filename = config.get_floppy(row)
-            if filename:
-                fsel.set_filename(filename)
-            elif path:
-                fsel.set_current_folder(path)
-            self.floppy.append(fsel)
-            
-            eject = create_button("Eject", self._eject, fsel)
-            box = gtk.HBox()
-            box.pack_start(fsel)
-            box.pack_start(eject, False, False)
+            fname = config.get_floppy(row)
+            fsel, box = factory.get(label, path, fname, gtk.FILE_CHOOSER_ACTION_OPEN)
             table_add_widget_row(table, row, label, box)
+            self.floppy.append(fsel)
             row += 1
 
+        protect = gtk.combo_box_new_text()
+        for text in config.get_protection_types():
+            protect.append_text(text)
+        protect.set_active(config.get_floppy_protection())
+        protect.set_tooltip_text("Write protect floppy image contents")
+        table_add_widget_row(table, row, "Write protection:", protect)
+
+        row += 1
+        fastfdc = gtk.CheckButton("Fast floppy access")
+        fastfdc.set_active(config.get_fastfdc())
+        fastfdc.set_tooltip_text("Can cause incompatibilities with some games/demos")
+        table_add_widget_row(table, row, None, fastfdc)
+
         table.show_all()
 
-    def _eject(self, widget, fsel):
-        fsel.unselect_all()
+        self.protect = protect
+        self.fastfdc = fastfdc
     
     def run(self, config):
         "run(config), show disk image dialog"
@@ -309,42 +319,138 @@ class DiskDialog(HatariUIDialog):
             config.lock_updates()
             for drive in range(2):
                 config.set_floppy(drive, self.floppy[drive].get_filename())
+            config.set_floppy_protection(self.protect.get_active())
+            config.set_fastfdc(self.fastfdc.get_active())
             config.flush_updates()
 
+
+# ----------------------------------
+# Hard disk dialog
+
+class HardDiskDialog(HatariUIDialog):
+    def _create_dialog(self, config):
+        table, self.dialog = create_table_dialog(self.parent, "Hard disks", 4, 4, "Set and reboot")
+        factory = FselAndEjectFactory()
+
+        row = 0
+        label = "ASCI HD image:"
+        path = config.get_acsi_image()
+        fsel, box = factory.get(label, None, path, gtk.FILE_CHOOSER_ACTION_OPEN)
+        table_add_widget_row(table, row, label, box, True)
+        self.acsi = fsel
+        row += 1
+
+        label = "IDE HD master image:"
+        path = config.get_idemaster_image()
+        fsel, box = factory.get(label, None, path, gtk.FILE_CHOOSER_ACTION_OPEN)
+        table_add_widget_row(table, row, label, box, True)
+        self.idemaster = fsel
+        row += 1
+
+        label = "IDE HD slave image:"
+        path = config.get_ideslave_image()
+        fsel, box = factory.get(label, None, path, gtk.FILE_CHOOSER_ACTION_OPEN)
+        table_add_widget_row(table, row, label, box, True)
+        self.ideslave = fsel
+        row += 1
+        
+        label = "GEMDOS drive directory:"
+        path = config.get_gemdos_dir()
+        fsel, box = factory.get(label, None, path, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
+        table_add_widget_row(table, row, label, box, True)
+        self.gemdos = fsel
+        row += 1
+
+        protect = gtk.combo_box_new_text()
+        for text in config.get_protection_types():
+            protect.append_text(text)
+        protect.set_tooltip_text("Write protect GEMDOS drive contents")
+        table_add_widget_row(table, row, "Write protection:", protect)
+        self.protect = protect
+
+        table.show_all()
     
+    def _get_config(self, config):
+        path = config.get_gemdos_dir()
+        if path:
+            self.gemdos.set_filename(path)
+        path = config.get_acsi_image()
+        if path:
+            self.acsi.set_filename(path)
+        path = config.get_idemaster_image()
+        if path:
+            self.idemaster.set_filename(path)
+        path = config.get_ideslave_image()
+        if path:
+            self.ideslave.set_filename(path)
+        self.protect.set_active(config.get_hd_protection())
+        
+    def _set_config(self, config):
+        config.lock_updates()
+        config.set_gemdos_dir(self.gemdos.get_filename())
+        config.set_acsi_image(self.acsi.get_filename())
+        config.set_idemaster_image(self.idemaster.get_filename())
+        config.set_ideslave_image(self.ideslave.get_filename())
+        config.set_hd_protection(self.protect.get_active())
+        config.flush_updates()
+
+    def run(self, config):
+        "run(config) -> bool, whether to reboot"
+        if not self.dialog:
+            self._create_dialog(config)
+
+        self._get_config(config)
+        response = self.dialog.run()
+        self.dialog.hide()
+        if response == gtk.RESPONSE_APPLY:
+            self._set_config(config)
+            return True
+        return False
+
+
 # ---------------------------
 # Display dialog
 
 class DisplayDialog(HatariUIDialog):
 
     def _create_dialog(self, config):
-        tips = gtk.Tooltips()
 
         skip = gtk.combo_box_new_text()
         for text in config.get_frameskip_names():
             skip.append_text(text)
         skip.set_active(config.get_frameskip())
-        tips.set_tip(skip, "Set how many frames are skipped")
+        skip.set_tooltip_text("Set how many frames are skipped")
 
-        zoom = gtk.CheckButton("Zoom ST-low")
-        zoom.set_active(config.get_zoom())
-        tips.set_tip(zoom, "Whether to double ST-low resolution")
+        maxw, maxh = config.get_max_size()
+        topw, toph = config.get_desktop_size()
+        maxadjw = gtk.Adjustment(maxw, 320, topw, 8, 40)
+        maxadjh = gtk.Adjustment(maxh, 200, toph, 8, 40)
+        scalew = gtk.HScale(maxadjw)
+        scaleh = gtk.HScale(maxadjh)
+        scalew.set_digits(0)
+        scaleh.set_digits(0)
+        scalew.set_tooltip_text("Preferred/maximum zoomed width")
+        scaleh.set_tooltip_text("Preferred/maximum zoomed height")
 
-        borders = gtk.CheckButton("Borders")
+        desktop = gtk.CheckButton("Keep desktop resolution (for Falcon/TT)")
+        desktop.set_active(config.get_desktop())
+        desktop.set_tooltip_text("Whether to keep desktop resolution in fullscreen and (try to) scale Atari screen by an integer factor instead")
+
+        borders = gtk.CheckButton("ST/STE overscan borders")
         borders.set_active(config.get_borders())
-        tips.set_tip(borders, "Whether to show overscan borders in low/mid-rez")
+        borders.set_tooltip_text("Whether to show overscan borders in ST/STE low/mid-rez. Visible border area is affected by max. zoom size")
 
-        spec512 = gtk.CheckButton("Spec512")
-        spec512.set_active(config.get_spec512threshold())
-        tips.set_tip(spec512, "Whether to support Spec512 (>16 colors at the same time)")
-
-        statusbar = gtk.CheckButton("Statusbar")
+        statusbar = gtk.CheckButton("Show statusbar")
         statusbar.set_active(config.get_statusbar())
-        tips.set_tip(statusbar, "Whether to show statusbar with floppy leds etc")
+        statusbar.set_tooltip_text("Whether to show statusbar with floppy leds etc")
 
-        led = gtk.CheckButton("Overlay led")
+        led = gtk.CheckButton("Show overlay led")
         led.set_active(config.get_led())
-        tips.set_tip(led, "Whether to show overlay drive led when statusbar isn't visible")
+        led.set_tooltip_text("Whether to show overlay drive led when statusbar isn't visible")
+
+        crop = gtk.CheckButton("Remove statusbar from screen capture")
+        crop.set_active(config.get_crop())
+        crop.set_tooltip_text("Whether to crop statusbar from screenshots and video recordings")
 
         dialog = gtk.Dialog("Display settings", self.parent,
             gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
@@ -353,20 +459,25 @@ class DisplayDialog(HatariUIDialog):
 
         dialog.vbox.add(gtk.Label("Frameskip:"))
         dialog.vbox.add(skip)
-        dialog.vbox.add(zoom)
+        dialog.vbox.add(gtk.Label("Max zoomed size:"))
+        dialog.vbox.add(scalew)
+        dialog.vbox.add(scaleh)
+        dialog.vbox.add(desktop)
         dialog.vbox.add(borders)
-        dialog.vbox.add(spec512)
         dialog.vbox.add(statusbar)
         dialog.vbox.add(led)
+        dialog.vbox.add(crop)
         dialog.vbox.show_all()
 
         self.dialog = dialog
         self.skip = skip
-        self.zoom = zoom
+        self.maxw = maxadjw
+        self.maxh = maxadjh
+        self.desktop = desktop
         self.borders = borders
-        self.spec512 = spec512
         self.statusbar = statusbar
         self.led = led
+        self.crop = crop
  
     def run(self, config):
         "run(config), show display dialog"
@@ -377,11 +488,12 @@ class DisplayDialog(HatariUIDialog):
         if response == gtk.RESPONSE_APPLY:
             config.lock_updates()
             config.set_frameskip(self.skip.get_active())
-            config.set_zoom(self.zoom.get_active())
+            config.set_max_size(self.maxw.get_value(), self.maxh.get_value())
+            config.set_desktop(self.desktop.get_active())
             config.set_borders(self.borders.get_active())
-            config.set_spec512threshold(self.spec512.get_active())
             config.set_statusbar(self.statusbar.get_active())
             config.set_led(self.led.get_active())
+            config.set_crop(self.crop.get_active())
             config.flush_updates()
 
 
@@ -390,7 +502,7 @@ class DisplayDialog(HatariUIDialog):
 
 class JoystickDialog(HatariUIDialog):
     def _create_dialog(self, config):
-        table, self.dialog = create_table_dialog(self.parent, "Joystick settings", 9)
+        table, self.dialog = create_table_dialog(self.parent, "Joystick settings", 9, 2)
         
         joy = 0
         self.joy = []
@@ -469,7 +581,7 @@ class PeripheralDialog(HatariUIDialog):
 class PathDialog(HatariUIDialog):
     def _create_dialog(self, config):
         paths = config.get_paths()
-        table, self.dialog = create_table_dialog(self.parent, "File path settings", len(paths))
+        table, self.dialog = create_table_dialog(self.parent, "File path settings", len(paths), 2)
         paths.sort()
         row = 0
         self.paths = []
@@ -509,24 +621,36 @@ class PathDialog(HatariUIDialog):
 class SoundDialog(HatariUIDialog):
 
     def _create_dialog(self, config):
-        combo = gtk.combo_box_entry_new_text()
-        for text in config.get_sound_values():
-            combo.append_text(text)
-        enabled, hz = config.get_sound()
+        enabled, curhz = config.get_sound()
+
         self.enabled = gtk.CheckButton("Sound enabled")
         self.enabled.set_active(enabled)
-        combo.child.set_text(hz)
-        box = gtk.HBox()
-        box.pack_start(gtk.Label("Sound frequency:"), False, False)
-        box.add(combo)
-        self.sound = combo.child
+
+        hz = gtk.combo_box_new_text()
+        for text in config.get_sound_values():
+            hz.append_text(text)
+        hz.set_active(curhz)
+        self.hz = hz
+
+        ymmixer = gtk.combo_box_new_text()
+        for text in config.get_ymmixer_types():
+            ymmixer.append_text(text)
+        ymmixer.set_active(config.get_ymmixer())
+        self.ymmixer = ymmixer
+
+        self.mic = gtk.CheckButton("Enable (Falcon) microphone")
+        self.mic.set_active(config.get_mic())
 
         dialog = gtk.Dialog("Sound settings", self.parent,
             gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
             (gtk.STOCK_APPLY,  gtk.RESPONSE_APPLY,
              gtk.STOCK_CANCEL,  gtk.RESPONSE_CANCEL))
         dialog.vbox.add(self.enabled)
-        dialog.vbox.add(box)
+        dialog.vbox.add(gtk.Label("Sound frequency::"))
+        dialog.vbox.add(hz)
+        dialog.vbox.add(gtk.Label("YM voices mixing method:"))
+        dialog.vbox.add(ymmixer)
+        dialog.vbox.add(self.mic)
         dialog.vbox.show_all()
         self.dialog = dialog
 
@@ -537,11 +661,13 @@ class SoundDialog(HatariUIDialog):
         response = self.dialog.run()
         self.dialog.hide()
         if response == gtk.RESPONSE_APPLY:
+            config.lock_updates()
+            config.set_mic(self.mic.get_active())
+            config.set_ymmixer(self.ymmixer.get_active())
             enabled = self.enabled.get_active()
-            hz1 = self.sound.get_text()
-            hz2 = config.set_sound(enabled, hz1)
-            if hz2 != hz1:
-                self.sound.set_text(hz2)
+            hz = self.hz.get_active()
+            config.set_sound(enabled, hz)
+            config.flush_updates()
         
 
 # ---------------------------
@@ -564,8 +690,8 @@ class TraceDialog(HatariUIDialog):
         "mfp_start",
         "mfp_read",
         "mfp_write",
-        "psg_write_reg",
-        "psg_write_data",
+        "psg_read",
+        "psg_write",
         "cpu_pairing",
         "cpu_disasm",
         "cpu_exception",
@@ -579,24 +705,33 @@ class TraceDialog(HatariUIDialog):
         "xbios",
         "gemdos",
         "vdi",
+        "aes",
         "io_read",
         "io_write",
-        "dmasound"
+        "dmasound",
+        "crossbar",
+        "videl",
+        "dsp_host_interface",
+        "dsp_host_command",
+        "dsp_host_ssi",
+        "dsp_interrupt",
+        "dsp_disasm",
+        "dsp_state"
     ]
     def __init__(self, parent):
-        self.savedpoints = "none"
+        self.savedpoints = None
         hbox1 = gtk.HBox()
         hbox1.add(create_button("Load", self._load_traces))
         hbox1.add(create_button("Clear", self._clear_traces))
         hbox1.add(create_button("Save", self._save_traces))
         hbox2 = gtk.HBox()
         vboxes = []
-        for idx in (0,1,2):
+        for idx in (0,1,2,3):
             vboxes.append(gtk.VBox())
             hbox2.add(vboxes[idx])
 
         count = 0
-        per_side = (len(self.tracepoints)+2)/3
+        per_side = (len(self.tracepoints)+3)/4
         self.tracewidgets = {}
         for trace in self.tracepoints:
             name = trace.replace("_", "-")
@@ -626,11 +761,13 @@ class TraceDialog(HatariUIDialog):
 
     def _set_traces(self, tracepoints):
         self._clear_traces()
+        if not tracepoints:
+            return
         for trace in tracepoints.split(","):
             if trace in self.tracewidgets:
                 self.tracewidgets[trace].set_active(True)
             else:
-                print "ERROR: unknown trace setting '%s'" % trace
+                print("ERROR: unknown trace setting '%s'" % trace)
 
     def _clear_traces(self, widget = None):
         for trace in self.tracepoints:
@@ -661,48 +798,69 @@ class TraceDialog(HatariUIDialog):
 # Machine dialog for settings needing reboot
 
 class MachineDialog(HatariUIDialog):
-    def _create_dialog(self, config):
-        table, self.dialog = create_table_dialog(self.parent, "Machine configuration", 9, "Set and reboot")
+    def _machine_cb(self, widget, data):
+        if not widget.get_active():
+            return
+        machine = data.lower()
+        if machine == "ste" or machine == "st":
+            self.clocks[0].set_active(True)
+            self.cpulevel.set_active(0)
+        elif machine == "falcon":
+            self.clocks[1].set_active(True)
+            self.dsps[2].set_active(True)
+            self.cpulevel.set_active(3)
+        elif machine == "tt":
+            self.clocks[2].set_active(True)
+            self.cpulevel.set_active(3)
 
+    def _create_dialog(self, config):
+        table, self.dialog = create_table_dialog(self.parent, "Machine configuration", 6, 4, "Set and reboot")
+        
         row = 0
-        combo = gtk.combo_box_new_text()
-        for text in config.get_machine_types():
-            combo.append_text(text)
-        self.machine = table_add_widget_row(table, row, "Machine type:", combo)
+        self.machines = table_add_radio_rows(table, row, "Machine:",
+                        config.get_machine_types(), self._machine_cb)
         row += 1
-        
-        combo = gtk.combo_box_new_text()
-        for text in config.get_monitor_types():
-            combo.append_text(text)
-        self.monitor = table_add_widget_row(table, row, "Monitor type:", combo)
+
+        self.dsps = table_add_radio_rows(table, row, "DSP type:", config.get_dsp_types())
         row += 1
-        
+
+        # start next table column
+        row = 0
+        table_set_col_offset(table, 2)
+        self.monitors = table_add_radio_rows(table, row, "Monitor:", config.get_monitor_types())
+        row += 1
+
+        self.clocks = table_add_radio_rows(table, row, "CPU clock:", config.get_cpuclock_types())
+        row += 1
+
+        # fullspan at bottom
+        fullspan = True
+
+        combo = gtk.combo_box_new_text()
+        for text in config.get_cpulevel_types():
+            combo.append_text(text)
+        self.cpulevel = table_add_widget_row(table, row, "CPU type:", combo, fullspan)
+        row += 1
+
         combo = gtk.combo_box_new_text()
         for text in config.get_memory_names():
             combo.append_text(text)
-        self.memory = table_add_widget_row(table, row, "Memory size:", combo)
+        self.memory = table_add_widget_row(table, row, "Memory:", combo, fullspan)
         row += 1
         
         label = "TOS image:"
         fsel = self._fsel(label, gtk.FILE_CHOOSER_ACTION_OPEN)
-        self.tos = table_add_widget_row(table, row, label, fsel)
-        row += 1
-        
-        label = "Harddisk:"
-        fsel = self._fsel(label, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
-        self.harddisk = table_add_widget_row(table, row, label, fsel)       
+        self.tos = table_add_widget_row(table, row, label, fsel, fullspan)
         row += 1
 
-        widget = gtk.CheckButton("Use harddisk")
-        self.usehd = table_add_widget_row(table, row, None, widget)
-        row += 1
-
-        widget = gtk.CheckButton("Compatible CPU")
-        self.compatible = table_add_widget_row(table, row, None, widget)
-        row += 1
-
-        widget = gtk.CheckButton("Patch Timer-D")
-        self.timerd = table_add_widget_row(table, row, None, widget)
+        vbox = gtk.VBox()
+        self.compatible = gtk.CheckButton("Compatible CPU")
+        self.rtc = gtk.CheckButton("Real-time clock")
+        self.timerd = gtk.CheckButton("Patch Timer-D")
+        vbox.add(self.compatible)
+        vbox.add(self.timerd)
+        vbox.add(self.rtc)
+        table_add_widget_row(table, row, "Misc.:", vbox, fullspan)
         row += 1
 
         table.show_all()
@@ -716,32 +874,38 @@ class MachineDialog(HatariUIDialog):
         return fsel
     
     def _get_config(self, config):
-        self.machine.set_active(config.get_machine())
-        self.monitor.set_active(config.get_monitor())
+        self.machines[config.get_machine()].set_active(True)
+        self.monitors[config.get_monitor()].set_active(True)
+        self.clocks[config.get_cpuclock()].set_active(True)
+        self.dsps[config.get_dsp()].set_active(True)
+        self.cpulevel.set_active(config.get_cpulevel())
         self.memory.set_active(config.get_memory())
         tos = config.get_tos()
-        hd = config.get_harddisk()
-        usehd = config.get_use_harddisk()
         if tos:
             self.tos.set_filename(tos)
-        if hd:
-            self.harddisk.set_filename(hd)
-        if usehd:
-            self.usehd.set_active(usehd)
         self.compatible.set_active(config.get_compatible())
         self.timerd.set_active(config.get_timerd())
+        self.rtc.set_active(config.get_rtc())
 
+    def _get_active_radio(self, radios):
+        idx = 0
+        for radio in radios:
+            if radio.get_active():
+                return idx
+            idx += 1
+        
     def _set_config(self, config):
         config.lock_updates()
-        config.set_machine(self.machine.get_active())
-        config.set_monitor(self.monitor.get_active())
+        config.set_machine(self._get_active_radio(self.machines))
+        config.set_monitor(self._get_active_radio(self.monitors))
+        config.set_cpuclock(self._get_active_radio(self.clocks))
+        config.set_dsp(self._get_active_radio(self.dsps))
+        config.set_cpulevel(self.cpulevel.get_active())
         config.set_memory(self.memory.get_active())
         config.set_tos(self.tos.get_filename())
-        # usehd has to be before before harddisk
-        config.set_use_harddisk(self.usehd.get_active())
-        config.set_harddisk(self.harddisk.get_filename())
         config.set_compatible(self.compatible.get_active())
         config.set_timerd(self.timerd.get_active())
+        config.set_rtc(self.rtc.get_active())
         config.flush_updates()
 
     def run(self, config):
