@@ -24,10 +24,13 @@ const char DlgSound_fileid[] = "Hatari dlgSound.c : " __DATE__ " " __TIME__;
 #define DLGSOUND_44KHZ      11
 #define DLGSOUND_48KHZ      12
 #define DLGSOUND_50KHZ      13
-#define DLGSOUND_RECNAME    17
-#define DLGSOUND_RECBROWSE  18
-#define DLGSOUND_RECORD     19
-#define DLGSOUND_EXIT       20
+#define DLGSOUND_MODEL      15
+#define DLGSOUND_TABLE      16
+#define DLGSOUND_LINEAR     17
+#define DLGSOUND_RECNAME    21
+#define DLGSOUND_RECBROWSE  22
+#define DLGSOUND_RECORD     23
+#define DLGSOUND_EXIT       24
 
 
 static char dlgRecordName[35];
@@ -36,29 +39,34 @@ static char dlgRecordName[35];
 /* The sound dialog: */
 static SGOBJ sounddlg[] =
 {
-	{ SGBOX, 0, 0, 0,0, 38,24, NULL },
-	{ SGBOX, 0, 0, 1,1, 36,11, NULL },
-	{ SGTEXT, 0, 0, 13,2, 13,1, "Sound options" },
-	{ SGCHECKBOX, 0, 0, 12,4, 14,1, "Enable sound" },
+	{ SGBOX,      0,0,  0, 0, 40,25, NULL },
+	{ SGBOX,      0,0,  1, 1, 38,13, NULL },
+	{ SGTEXT,     0,0,  4, 2, 13,1, "SOUND" },
+	{ SGCHECKBOX, 0,0, 14, 2, 14,1, "Enabled" },
 
-	{ SGTEXT, 0, 0, 11,6, 14,1, "Playback quality:" },
-	{ SGRADIOBUT, 0, 0, 2,8, 10,1, "11025 Hz" },
-	{ SGRADIOBUT, 0, 0, 2,9, 10,1, "12517 Hz" },
-	{ SGRADIOBUT, 0, 0, 2,10, 10,1, "16000 Hz" },
-	{ SGRADIOBUT, 0, 0, 14,8, 10,1, "22050 Hz" },
-	{ SGRADIOBUT, 0, 0, 14,9, 10,1, "25033 Hz" },
-	{ SGRADIOBUT, 0, 0, 14,10, 10,1, "32000 Hz" },
-	{ SGRADIOBUT, 0, 0, 26,8, 10,1, "44100 Hz" },
-	{ SGRADIOBUT, 0, 0, 26,9, 10,1, "48000 Hz" },
-	{ SGRADIOBUT, 0, 0, 26,10, 10,1, "50066 Hz" },
+	{ SGTEXT,     0,0,  4, 4, 14,1, "Playback quality:" },
+	{ SGRADIOBUT, 0,0,  2, 6, 10,1, "11025 Hz" },
+	{ SGRADIOBUT, 0,0,  2, 7, 10,1, "12517 Hz" },
+	{ SGRADIOBUT, 0,0,  2, 8, 10,1, "16000 Hz" },
+	{ SGRADIOBUT, 0,0, 15, 6, 10,1, "22050 Hz" },
+	{ SGRADIOBUT, 0,0, 15, 7, 10,1, "25033 Hz" },
+	{ SGRADIOBUT, 0,0, 15, 8, 10,1, "32000 Hz" },
+	{ SGRADIOBUT, 0,0, 28, 6, 10,1, "44100 Hz" },
+	{ SGRADIOBUT, 0,0, 28, 7, 10,1, "48000 Hz" },
+	{ SGRADIOBUT, 0,0, 28, 8, 10,1, "50066 Hz" },
 
-	{ SGBOX, 0, 0, 1,13, 36,8, NULL },
-	{ SGTEXT, 0, 0, 13,14, 14,1, "Capture YM/WAV" },
-	{ SGTEXT, 0, 0, 2,16, 26,1, "File name (*.wav or *.ym):" },
-	{ SGTEXT, 0, 0, 2,17, 34,1, dlgRecordName },
-	{ SGBUTTON, 0, 0, 30,16, 6,1, "Browse" },
-	{ SGBUTTON, 0, 0, 12,19, 16,1, NULL },
-	{ SGBUTTON, SG_DEFAULT, 0, 10,22, 20,1, "Back to main menu" },
+	{ SGTEXT,     0,0,  4,10, 10,1, "YM voices mixing:" },
+	{ SGRADIOBUT, 0,0,  2,12, 10,1, "Math model" },
+	{ SGRADIOBUT, 0,0, 15,12, 10,1, "ST table" },
+	{ SGRADIOBUT, 0,0, 28,12, 10,1, "Linear" },
+
+	{ SGBOX,      0,0,  1,15, 38,7, NULL },
+	{ SGTEXT,     0,0, 13,16, 14,1, "Capture YM/WAV" },
+	{ SGTEXT,     0,0,  2,17, 26,1, "File name (*.wav / *.ym):" },
+	{ SGTEXT,     0,0,  2,18, 34,1, dlgRecordName },
+	{ SGBUTTON,   0,0, 28,17,  8,1, " Browse " },
+	{ SGBUTTON,   0,0, 12,20, 16,1, NULL },
+	{ SGBUTTON, SG_DEFAULT, 0, 10,23, 20,1, "Back to main menu" },
 	{ -1, 0, 0, 0,0, 0,0, NULL }
 };
 
@@ -78,9 +86,9 @@ static const int nSoundFreqs[] =
 
 
 /*-----------------------------------------------------------------------*/
-/*
-  Show and process the sound dialog.
-*/
+/**
+ * Show and process the sound dialog.
+ */
 void Dialog_SoundDlg(void)
 {
 	int but, i;
@@ -97,7 +105,7 @@ void Dialog_SoundDlg(void)
 	for (i = DLGSOUND_11KHZ; i <= DLGSOUND_50KHZ; i++)
 		sounddlg[i].state &= ~SG_SELECTED;
 
-	for (i = 0; i < DLGSOUND_50KHZ-DLGSOUND_11KHZ; i++)
+	for (i = 0; i <= DLGSOUND_50KHZ-DLGSOUND_11KHZ; i++)
 	{
 		if (ConfigureParams.Sound.nPlaybackFreq > nSoundFreqs[i]-500
 		    && ConfigureParams.Sound.nPlaybackFreq < nSoundFreqs[i]+500)
@@ -107,6 +115,17 @@ void Dialog_SoundDlg(void)
 		}
 	}
 
+	sounddlg[DLGSOUND_MODEL].state &= ~SG_SELECTED;
+	sounddlg[DLGSOUND_TABLE].state &= ~SG_SELECTED;
+	sounddlg[DLGSOUND_LINEAR].state &= ~SG_SELECTED;
+	if (ConfigureParams.Sound.YmVolumeMixing == YM_MODEL_MIXING)
+		sounddlg[DLGSOUND_MODEL].state |= SG_SELECTED;
+	else
+	if (ConfigureParams.Sound.YmVolumeMixing == YM_TABLE_MIXING)
+		sounddlg[DLGSOUND_TABLE].state |= SG_SELECTED;
+	else
+		sounddlg[DLGSOUND_LINEAR].state |= SG_SELECTED;
+
 	File_ShrinkName(dlgRecordName, ConfigureParams.Sound.szYMCaptureFileName, sounddlg[DLGSOUND_RECNAME].w);
 
 	if ( Sound_AreWeRecording() )
@@ -159,4 +178,12 @@ void Dialog_SoundDlg(void)
 			break;
 		}
 	}
+
+	if (sounddlg[DLGSOUND_MODEL].state & SG_SELECTED)
+		ConfigureParams.Sound.YmVolumeMixing = YM_MODEL_MIXING;
+	else
+	if (sounddlg[DLGSOUND_TABLE].state & SG_SELECTED)
+		ConfigureParams.Sound.YmVolumeMixing = YM_TABLE_MIXING;
+	else
+		ConfigureParams.Sound.YmVolumeMixing = YM_LINEAR_MIXING;
 }
