@@ -8,10 +8,13 @@
 #ifndef HATARI_CYCLES_H
 #define HATARI_CYCLES_H
 
+#include <stdbool.h>
+
 enum
 {
 	CYCLES_COUNTER_SOUND,
 	CYCLES_COUNTER_VIDEO,
+	CYCLES_COUNTER_CPU,
 
 	CYCLES_COUNTER_MAX
 };
@@ -20,8 +23,10 @@ enum
 extern int nCyclesMainCounter;
 
 extern int CurrentInstrCycles;
+extern int MovepByteNbr;
 
 
+extern void Cycles_MemorySnapShot_Capture(bool bSave);
 extern void Cycles_SetCounter(int nId, int nValue);
 extern int Cycles_GetCounter(int nId);
 extern int Cycles_GetCounterOnReadAccess(int nId);
