@@ -11,7 +11,7 @@
 
 /* Name and version for window title: */
 //#define PROG_NAME "Hatari devel (" __DATE__ ")"
-#define PROG_NAME "Hatari v1.3.1"
+#define PROG_NAME "Hatari v1.6.1"
 
 #include "config.h"
 
@@ -47,16 +47,18 @@
 #define SIZE_WORD  2
 #define SIZE_LONG  4
 
+/* The 8 MHz CPU frequency */
+#define CPU_FREQ   8012800
 
 extern bool bQuitProgram;
-extern Uint32 nRunVBLs;
 
-extern void Main_MemorySnapShot_Capture(bool bSave);
 extern bool Main_PauseEmulation(bool visualize);
 extern bool Main_UnPauseEmulation(void);
 extern void Main_RequestQuit(void);
+extern void Main_SetRunVBLs(Uint32 vbls);
 extern void Main_WaitOnVbl(void);
 extern void Main_WarpMouse(int x, int y);
 extern void Main_EventHandler(void);
+extern void Main_SetTitle(const char *title);
 
 #endif /* ifndef HATARI_MAIN_H */
