@@ -9,6 +9,7 @@
 const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
 
 #include <time.h>
+#include <errno.h>
 #include <SDL.h>
 
 #include "main.h"
@@ -33,28 +34,35 @@ const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
 #include "paths.h"
 #include "printer.h"
 #include "reset.h"
+#include "resolution.h"
 #include "rs232.h"
 #include "screen.h"
 #include "sdlgui.h"
 #include "shortcut.h"
 #include "sound.h"
+#include "dmaSnd.h"
 #include "statusbar.h"
 #include "stMemory.h"
 #include "str.h"
 #include "tos.h"
 #include "video.h"
+#include "avi_record.h"
+#include "debugui.h"
+#include "clocks_timings.h"
 
 #include "hatari-glue.h"
 
 #include "falcon/hostscreen.h"
-#if ENABLE_DSP_EMU
 #include "falcon/dsp.h"
+
+#if HAVE_GETTIMEOFDAY
+#include <sys/time.h>
 #endif
 
 
 bool bQuitProgram = false;                /* Flag to quit program cleanly */
 
-Uint32 nRunVBLs;                          /* Whether and how many VBLS to run before exit */
+static Uint32 nRunVBLs;                   /* Whether and how many VBLS to run before exit */
 static Uint32 nFirstMilliTick;            /* Ticks when VBL counting started */
 static Uint32 nVBLCount;                  /* Frame count */
 
@@ -63,32 +71,6 @@ static bool bAccurateDelays;              /* Host system has an accurate SDL_Del
 static bool bIgnoreNextMouseMotion = false;  /* Next mouse motion will be ignored (needed after SDL_WarpMouse) */
 
 
-/*-----------------------------------------------------------------------*/
-/**
- * Save/Restore snapshot of local variables('MemorySnapShot_Store' handles type)
- */
-void Main_MemorySnapShot_Capture(bool bSave)
-{
-	int nBytes;
-
-	/* Save/Restore details */
-	/* Only save/restore area of memory machine ie set to, eg 1Mb */
-	if (bSave)
-	{
-		nBytes = STRamEnd;
-		MemorySnapShot_Store(&nBytes, sizeof(nBytes));
-		MemorySnapShot_Store(STRam, nBytes);
-	}
-	else
-	{
-		MemorySnapShot_Store(&nBytes, sizeof(nBytes));
-		MemorySnapShot_Store(STRam, nBytes);
-	}
-	/* And Cart/TOS/Hardware area */
-	MemorySnapShot_Store(&RomMem[0xE00000], 0x200000);
-}
-
-
 /*-----------------------------------------------------------------------*/
 /**
  * Return current time as millisecond for performance measurements.
@@ -116,10 +98,63 @@ static Uint32 Main_GetTicks(void)
 	return ticks_to_msec * fields.tms_utime;
 }
 #else
-#define Main_GetTicks SDL_GetTicks
+# warning "times() function missing, using inaccurate SDL_GetTicks() instead."
+# define Main_GetTicks SDL_GetTicks
 #endif
 
 
+//#undef HAVE_GETTIMEOFDAY
+//#undef HAVE_NANOSLEEP
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Return a time counter in micro seconds.
+ * If gettimeofday is available, we use it directly, else we convert the
+ * return of SDL_GetTicks in micro sec.
+ */
+
+static Sint64	Time_GetTicks ( void )
+{
+        Sint64		ticks_micro;
+
+#if HAVE_GETTIMEOFDAY
+        struct timeval	now;
+        gettimeofday ( &now , NULL );
+        ticks_micro = (Sint64)now.tv_sec * 1000000 + now.tv_usec;
+#else
+	ticks_micro = (Sint64)SDL_GetTicks() * 1000;		/* milli sec -> micro sec */
+#endif
+
+	return ticks_micro;
+}
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Sleep for a given number of micro seconds.
+ * If nanosleep is available, we use it directly, else we use SDL_Delay
+ * (which is portable, but less accurate as is uses milli-seconds)
+ */
+
+static void	Time_Delay ( Sint64 ticks_micro )
+{
+#if HAVE_NANOSLEEP
+	struct timespec	ts;
+	int		ret;
+	ts.tv_sec = ticks_micro / 1000000;
+	ts.tv_nsec = (ticks_micro % 1000000) * 1000;	/* micro sec -> nano sec */
+	/* wait until all the delay is elapsed, including possible interruptions by signals */
+	do
+	{
+                errno = 0;
+                ret = nanosleep(&ts, &ts);
+	} while ( ret && ( errno == EINTR ) );		/* keep on sleeping if we were interrupted */
+#else
+	SDL_Delay ( (Uint32)(ticks_micro / 1000) ) ;	/* micro sec -> milli sec */
+#endif
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Pause emulation, stop sound.  'visualize' should be set true,
@@ -145,7 +180,7 @@ bool Main_PauseEmulation(bool visualize)
 			current = (1000.0 * nVBLCount) / interval;
 			printf("SPEED: %.1f VBL/s (%d/%.1fs), diff=%.1f%%\n",
 			       current, nVBLCount, interval/1000.0,
-			       previous ? 100*(current-previous)/previous : 0.0);
+			       previous>0.0 ? 100*(current-previous)/previous : 0.0);
 			nVBLCount = nFirstMilliTick = 0;
 			previous = current;
 		}
@@ -172,7 +207,7 @@ bool Main_UnPauseEmulation(void)
 	if ( bEmulationActive )
 		return false;
 
-	Sound_ResetBufferIndex();
+	Sound_BufferIndexNeedReset = true;
 	Audio_EnableAudio(ConfigureParams.Sound.bEnableSound);
 	bEmulationActive = true;
 
@@ -213,6 +248,18 @@ void Main_RequestQuit(void)
 	}
 }
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Set how many VBLs Hatari should run, from the moment this function
+ * is called.
+ */
+void Main_SetRunVBLs(Uint32 vbls)
+{
+	fprintf(stderr, "Exit after %d VBLs.\n", vbls);
+	nRunVBLs = vbls;
+	nVBLCount = 0;
+}
+
 /*-----------------------------------------------------------------------*/
 /**
  * This function waits on each emulated VBL to synchronize the real time
@@ -221,42 +268,48 @@ void Main_RequestQuit(void)
  * are very inaccurate on some systems like Linux 2.4 or Mac OS X (they can only
  * wait for a multiple of 10ms due to the scheduler on these systems), so we have
  * to "busy wait" there to get an accurate timing.
+ * All times are expressed as micro seconds, to avoid too much rounding error.
  */
 void Main_WaitOnVbl(void)
 {
-	int nCurrentMilliTicks;
-	static int nDestMilliTicks = 0;
-	int nFrameDuration;
-	signed int nDelay;
+	Sint64 CurrentTicks;
+	static Sint64 DestTicks = 0;
+	Sint64 FrameDuration_micro;
+	Sint64 nDelay;
 
-	nCurrentMilliTicks = SDL_GetTicks();
+	nVBLCount++;
+	if (nRunVBLs &&	nVBLCount >= nRunVBLs)
+	{
+		/* show VBLs/s */
+		Main_PauseEmulation(true);
+		exit(0);
+	}
 
-	nFrameDuration = 1000/nScreenRefreshRate;
-	nDelay = nDestMilliTicks - nCurrentMilliTicks;
+//	FrameDuration_micro = (Sint64) ( 1000000.0 / nScreenRefreshRate + 0.5 );	/* round to closest integer */
+	FrameDuration_micro = ClocksTimings_GetVBLDuration_micro ( ConfigureParams.System.nMachineType , nScreenRefreshRate );
+	CurrentTicks = Time_GetTicks();
+
+	if ( DestTicks == 0 )					/* first call, init DestTicks */
+		DestTicks = CurrentTicks + FrameDuration_micro;
+
+	nDelay = DestTicks - CurrentTicks;
 
 	/* Do not wait if we are in fast forward mode or if we are totally out of sync */
 	if (ConfigureParams.System.bFastForward == true
-	        || nDelay < -4*nFrameDuration)
+	        || nDelay < -4*FrameDuration_micro)
 	{
 		if (ConfigureParams.System.bFastForward == true)
 		{
-			nVBLCount += 1;
 			if (!nFirstMilliTick)
 				nFirstMilliTick = Main_GetTicks();
-			else if (nRunVBLs && nVBLCount >= nRunVBLs)
-			{
-				/* show VBLs/s */
-				Main_PauseEmulation(true);
-				exit(0);
-			}
 		}
 		if (nFrameSkips < ConfigureParams.Screen.nFrameSkips)
 		{
 			nFrameSkips += 1;
 			// Log_Printf(LOG_DEBUG, "Increased frameskip to %d\n", nFrameSkips);
 		}
-		/* Only update nDestMilliTicks for next VBL */
-		nDestMilliTicks = nCurrentMilliTicks + nFrameDuration;
+		/* Only update DestTicks for next VBL */
+		DestTicks = CurrentTicks + FrameDuration_micro;
 		return;
 	}
 	/* If automatic frameskip is enabled and delay's more than twice
@@ -264,7 +317,7 @@ void Main_WaitOnVbl(void)
 	 */
 	if (nFrameSkips > 0
 	    && ConfigureParams.Screen.nFrameSkips >= AUTO_FRAMESKIP_LIMIT
-	    && 2*nDelay > nFrameDuration/nFrameSkips)
+	    && 2*nDelay > FrameDuration_micro/nFrameSkips)
 	{
 		nFrameSkips -= 1;
 		// Log_Printf(LOG_DEBUG, "Decreased frameskip to %d\n", nFrameSkips);
@@ -273,25 +326,26 @@ void Main_WaitOnVbl(void)
 	if (bAccurateDelays)
 	{
 		/* Accurate sleeping is possible -> use SDL_Delay to free the CPU */
-		if (nDelay > 1)
-			SDL_Delay(nDelay - 1);
+		if (nDelay > 1000)
+			Time_Delay(nDelay - 1000);
 	}
 	else
 	{
 		/* No accurate SDL_Delay -> only wait if more than 5ms to go... */
-		if (nDelay > 5)
-			SDL_Delay(nDelay<10 ? nDelay-1 : 9);
+		if (nDelay > 5000)
+			Time_Delay(nDelay<10000 ? nDelay-1000 : 9000);
 	}
 
 	/* Now busy-wait for the right tick: */
 	while (nDelay > 0)
 	{
-		nCurrentMilliTicks = SDL_GetTicks();
-		nDelay = nDestMilliTicks - nCurrentMilliTicks;
+		CurrentTicks = Time_GetTicks();
+		nDelay = DestTicks - CurrentTicks;
 	}
 
-	/* Update nDestMilliTicks for next VBL */
-	nDestMilliTicks += nFrameDuration;
+//printf ( "tick %lld\n" , CurrentTicks );
+	/* Update DestTicks for next VBL */
+	DestTicks += FrameDuration_micro;
 }
 
 
@@ -492,6 +546,18 @@ void Main_EventHandler(void)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Set Hatari window title. Use NULL for default
+ */
+void Main_SetTitle(const char *title)
+{
+	if (title)
+		SDL_WM_SetCaption(title, "Hatari");
+	else
+		SDL_WM_SetCaption(PROG_NAME, "Hatari");
+}
+
 /*-----------------------------------------------------------------------*/
 /**
  * Initialise emulation
@@ -508,42 +574,35 @@ static void Main_Init(void)
 
 	/* Init SDL's video subsystem. Note: Audio and joystick subsystems
 	   will be initialized later (failures there are not fatal). */
-	if (SDL_Init(SDL_INIT_VIDEO) < 0)
+	if (SDL_Init(SDL_INIT_VIDEO | Opt_GetNoParachuteFlag()) < 0)
 	{
 		fprintf(stderr, "Could not initialize the SDL library:\n %s\n", SDL_GetError() );
 		exit(-1);
 	}
-
+	ClocksTimings_InitMachine ( ConfigureParams.System.nMachineType );
+	Resolution_Init();
 	SDLGui_Init();
 	Printer_Init();
 	RS232_Init();
 	Midi_Init();
+	Control_CheckUpdates();       /* enable window embedding? */
 	Screen_Init();
+	Main_SetTitle(NULL);
 	HostScreen_Init();
-#if ENABLE_DSP_EMU
-	if (ConfigureParams.System.nDSPType == DSP_TYPE_EMU)
-	{
-		DSP_Init();
-	}
-#endif
+	DSP_Init();
 	Floppy_Init();
-	Init680x0();                  /* Init CPU emulation */
+	M68000_Init();                /* Init CPU emulation */
 	Audio_Init();
+	DmaSnd_Init();
 	Keymap_Init();
 
 	/* Init HD emulation */
-	if (ConfigureParams.HardDisk.bUseHardDiskImage)
-	{
-		char *szHardDiskImage = ConfigureParams.HardDisk.szHardDiskImage;
-		if (HDC_Init(szHardDiskImage))
-			printf("Hard drive image %s mounted.\n", szHardDiskImage);
-		else
-			printf("Couldn't open HD file: %s, or no partitions\n", szHardDiskImage);
-	}
+	HDC_Init();
 	Ide_Init();
 	GemDOS_Init();
 	if (ConfigureParams.HardDisk.bUseHardDiskDirectories)
 	{
+		/* uses variables set by HDC_Init()! */
 		GemDOS_InitDrives();
 	}
 
@@ -564,6 +623,9 @@ static void Main_Init(void)
 	NvRam_Init();
 	Joy_Init();
 	Sound_Init();
+	
+	/* done as last, needs CPU & DSP running... */
+	DebugUI_Init();
 }
 
 
@@ -588,12 +650,7 @@ static void Main_UnInit(void)
 		Sound_EndRecording();
 	Audio_UnInit();
 	SDLGui_UnInit();
-#if ENABLE_DSP_EMU
-	if (ConfigureParams.System.nDSPType == DSP_TYPE_EMU)
-	{
-		DSP_UnInit();
-	}
-#endif
+	DSP_UnInit();
 	HostScreen_UnInit();
 	Screen_UnInit();
 	Exit680x0();
@@ -649,6 +706,9 @@ static void Main_StatusbarSetup(void)
 	if (name)
 	{
 		char message[24], *keyname;
+#ifdef _MUDFLAP
+		__mf_register(name, 32, __MF_TYPE_GUESS, "SDL keyname");
+#endif
 		keyname = Str_ToUpper(strdup(name));
 		snprintf(message, sizeof(message), "Press %s for Options", keyname);
 		free(keyname);
@@ -680,7 +740,7 @@ int main(int argc, char *argv[])
 	Main_LoadInitialConfig();
 
 	/* Check for any passed parameters */
-	if (!Opt_ParseParameters(argc, (const char**)argv))
+	if (!Opt_ParseParameters(argc, (const char * const *)argv))
 	{
 		return 1;
 	}
@@ -691,11 +751,13 @@ int main(int argc, char *argv[])
 	Win_OpenCon();
 #endif
 
-	/* Needed on maemo but useful also with normal X11 window managers
-	 * for window grouping when you have multiple Hatari SDL windows open
-	 */
 #if HAVE_SETENV
+	/* Needed on maemo but useful also with normal X11 window managers for
+	 * window grouping when you have multiple Hatari SDL windows open */
 	setenv("SDL_VIDEO_X11_WMCLASS", "hatari", 1);
+
+	/* Needed for proper behavior of Caps Lock on some systems */
+	setenv("SDL_DISABLE_LOCK_KEYS", "1", 1);
 #endif
 
 	/* Init emulator system */
@@ -707,10 +769,25 @@ int main(int argc, char *argv[])
 	/* Check if SDL_Delay is accurate */
 	Main_CheckForAccurateDelays();
 
+	if ( AviRecordOnStartup )	/* Immediatly starts avi recording ? */
+		Avi_StartRecording ( ConfigureParams.Video.AviRecordFile , ConfigureParams.Screen.bCrop ,
+			ConfigureParams.Video.AviRecordFps == 0 ?
+				ClocksTimings_GetVBLPerSec ( ConfigureParams.System.nMachineType , nScreenRefreshRate ) :
+				(Uint32)ConfigureParams.Video.AviRecordFps << CLOCKS_TIMINGS_SHIFT_VBL ,
+			1 << CLOCKS_TIMINGS_SHIFT_VBL ,
+			ConfigureParams.Video.AviRecordVcodec );
+
 	/* Run emulation */
 	Main_UnPauseEmulation();
 	M68000_Start();                 /* Start emulation */
 
+	if (bRecordingAvi)
+	{
+		/* cleanly close the avi file */
+		Statusbar_AddMessage("Finishing AVI file...", 100);
+		Statusbar_Update(sdlscrn);
+		Avi_StopRecording();
+	}
 	/* Un-init emulation system */
 	Main_UnInit();
 
