@@ -35,7 +35,6 @@ static FILE *pPrinterHandle;
 
 
 /* internal functions */
-static void Printer_EmptyFile(void);
 static void Printer_ResetInternalBuffer(void);
 static bool Printer_EmptyInternalBuffer(void);
 static void Printer_AddByteToInternalBuffer(Uint8 Byte);
@@ -105,7 +104,7 @@ void Printer_CloseAllConnections(void)
 	pPrinterHandle = File_Close(pPrinterHandle);
 
 	/* Signal finished with printing */
-	bConnectedPrinter = FALSE;
+	bConnectedPrinter = false;
 }
 
 
@@ -119,27 +118,6 @@ static void Printer_ResetInternalBuffer(void)
 }
 
 
-/*-----------------------------------------------------------------------*/
-/**
- * Empty to file on disk.
- */
-static void Printer_EmptyFile(void)
-{
-	/* Do have file open? */
-	if (pPrinterHandle)
-	{
-		/* Write bytes out */
-		if (fwrite((Uint8 *)PrinterBuffer,sizeof(Uint8),nPrinterBufferChars,pPrinterHandle) < nPrinterBufferChars)
-		{
-			/* we wrote less then all chars in the buffer --> ERROR */
-			fprintf(stderr,"Printer_EmptyFile(): ERROR not all chars were written\n");
-		}
-		/* Reset */
-		Printer_ResetInternalBuffer();
-	}
-}
-
-
 /*-----------------------------------------------------------------------*/
 /**
  * Empty Printer Buffer
@@ -150,12 +128,26 @@ static bool Printer_EmptyInternalBuffer(void)
 	if (nPrinterBufferChars > 0)
 	{
 		if (pPrinterHandle)
-			Printer_EmptyFile();
+		{
+			size_t n;
+			/* Write bytes out */
+			n = fwrite((Uint8 *)PrinterBuffer, sizeof(Uint8),
+			           nPrinterBufferChars, pPrinterHandle);
+			if (n < nPrinterBufferChars)
+			{
+				/* we wrote less then expected! */
+				fprintf(stderr, "Printer_EmptyInternalBuffer():"
+					"ERROR not all chars were written\n");
+			}
+		}
 
-		return TRUE;
+		/* Reset */
+		Printer_ResetInternalBuffer();
+
+		return true;
 	}
 	/* Nothing to do */
-	return FALSE;
+	return false;
 }
 
 
@@ -176,14 +168,14 @@ static void Printer_AddByteToInternalBuffer(Uint8 Byte)
 /*-----------------------------------------------------------------------*/
 /**
  * Pass byte from emulator to printer.  Opens the printer file appending
- * if it isn't already open. Returns FALSE if connection to "printer"
+ * if it isn't already open. Returns false if connection to "printer"
  * failed
  */
 bool Printer_TransferByteTo(Uint8 Byte)
 {
 	/* Do we want to output to a printer/file? */
 	if (!ConfigureParams.Printer.bEnablePrinting)
-		return FALSE;   /* Failed if printing disabled */
+		return false;   /* Failed if printing disabled */
 
 	/* Have we made a connection to our printer/file? */
 	if (!bConnectedPrinter)
@@ -201,10 +193,10 @@ bool Printer_TransferByteTo(Uint8 Byte)
 	{
 		/* Add byte to our buffer. */
 		Printer_AddByteToInternalBuffer(Byte);
-		return TRUE;    /* OK */
+		return true;    /* OK */
 	}
 	else
-		return FALSE;   /* Failed */
+		return false;   /* Failed */
 }
 
 
