@@ -57,6 +57,11 @@
 /*			always return 0xff (tested on STF).				*/
 /*			When PSGRegisterSelect > 15, reads to $ff8800 also return 0xff.	*/
 /* 2009/01/24	[NP]	Remove redundant test, as movep implies SIZE_BYTE access.	*/
+/* 2011/10/30	[NP]	There's a special case when reading a register from $ff8800 :	*/
+/*			if the register number was not changed since the last write,	*/
+/*			then we must return the value that was written to $ff8802	*/
+/*			without masking the unused bit (fix the game Murders In Venice,	*/
+/*			which expects to read $10 from reg 3).				*/
 
 
 /* Emulating wait states when accessing $ff8800/01/02/03 with different 'move' variants	*/
@@ -103,7 +108,7 @@
 
 
 
-const char PSG_rcsid[] = "Hatari $Id: psg.c,v 1.34 2008/11/03 20:46:05 thothy Exp $";
+const char PSG_fileid[] = "Hatari psg.c : " __DATE__ " " __TIME__;
 
 #include "main.h"
 #include "configuration.h"
@@ -118,13 +123,15 @@ const char PSG_rcsid[] = "Hatari $Id: psg.c,v 1.34 2008/11/03 20:46:05 thothy Ex
 #if ENABLE_DSP_EMU
 #include "falcon/dsp.h"
 #endif
+#include "screen.h"
 #include "video.h"
 #include "statusbar.h"
 #include "mfp.h"
 
 
-Uint8 PSGRegisterSelect;        /* 0xff8800 (read/write) */
-Uint8 PSGRegisters[16];         /* Register in PSG, see PSG_REG_xxxx */
+Uint8 PSGRegisterSelect;        /* Write to 0xff8800 sets the register number used in read/write accesses */
+Uint8 PSGRegisterReadData;	/* Value returned when reading from 0xff8800 */
+Uint8 PSGRegisters[16];         /* Registers in PSG, see PSG_REG_xxxx */
 
 static unsigned int LastStrobe=0; /* Falling edge of Strobe used for printer */
 
@@ -135,8 +142,14 @@ static unsigned int LastStrobe=0; /* Falling edge of Strobe used for printer */
  */
 void PSG_Reset(void)
 {
+        int     i;
+
 	PSGRegisterSelect = 0;
+	PSGRegisterReadData = 0;
 	memset(PSGRegisters, 0, sizeof(PSGRegisters));
+        for ( i=0 ; i<14 ; i++ )		/* Update sound's emulation registers */
+		Sound_WriteReg ( i , 0 );
+
 	LastStrobe=0;
 }
 
@@ -149,6 +162,7 @@ void PSG_MemorySnapShot_Capture(bool bSave)
 {
 	/* Save/Restore details */
 	MemorySnapShot_Store(&PSGRegisterSelect, sizeof(PSGRegisterSelect));
+	MemorySnapShot_Store(&PSGRegisterReadData, sizeof(PSGRegisterReadData));
 	MemorySnapShot_Store(PSGRegisters, sizeof(PSGRegisters));
 	MemorySnapShot_Store(&LastStrobe, sizeof(LastStrobe));
 }
@@ -167,13 +181,19 @@ void PSG_Set_SelectRegister(Uint8 val)
 	/* read/write to ff8802 when PSGRegisterSelect >= 16 */
 	PSGRegisterSelect = val;
 
-	if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_PSG_WRITE ) )
-	  {
-	    int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);;
-	    int nLineCycles = nFrameCycles % nCyclesPerLine;
-	    HATARI_TRACE_PRINT ( "ym write reg=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n" ,
-		PSGRegisterSelect, nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles );
-	  }
+	/* When address register is changed, a read from $ff8800 should */
+	/* return the masked value of the register. We set the value here */
+	/* to be returned in case PSG_Get_DataRegister is called */
+	PSGRegisterReadData = PSGRegisters[PSGRegisterSelect];
+
+	if (LOG_TRACE_LEVEL(TRACE_PSG_WRITE))
+	{
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		LOG_TRACE_PRINT("ym write reg=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
+		                PSGRegisterSelect, FrameCycles, LineCycles, HblCounterVideo,
+		                M68000_GetPC(), CurrentInstrCycles);
+	}
 }
 
 
@@ -183,8 +203,6 @@ void PSG_Set_SelectRegister(Uint8 val)
  */
 Uint8 PSG_Get_DataRegister(void)
 {
-	M68000_WaitState(4);
-
 	/* Is a valid PSG register currently selected ? */
 	if ( PSGRegisterSelect >= 16 )
 		return 0xff;				/* not valid, return 0xff */
@@ -216,7 +234,7 @@ Uint8 PSG_Get_DataRegister(void)
 	}
 
 	/* Read data last selected by register */
-	return PSGRegisters[PSGRegisterSelect];
+	return PSGRegisterReadData;
 }
 
 
@@ -226,20 +244,25 @@ Uint8 PSG_Get_DataRegister(void)
  */
 void PSG_Set_DataRegister(Uint8 val)
 {
-	if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_PSG_WRITE ) )
-	  {
-	    int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);;
-	    int nLineCycles = nFrameCycles % nCyclesPerLine;
-	    HATARI_TRACE_PRINT ( "ym write data reg=0x%x val=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n" ,
-		PSGRegisterSelect, val, nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles );
-	  }
+	if (LOG_TRACE_LEVEL(TRACE_PSG_WRITE))
+	{
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		LOG_TRACE_PRINT("ym write data reg=0x%x val=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
+		                PSGRegisterSelect, val, FrameCycles, LineCycles,
+		                HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
+	}
 
 	/* Is a valid PSG register currently selected ? */
 	if ( PSGRegisterSelect >= 16 )
 		return;					/* not valid, ignore write and do nothing */
 
 	/* Create samples up until this point with current values */
-	Sound_Update();
+	Sound_Update(false);
+
+	/* When a read is made from $ff8800 without changing PSGRegisterSelect, we should return */
+	/* the non masked value. */
+	PSGRegisterReadData = val;			/* store non masked value for PSG_Get_DataRegister */
 
 	/* Copy value to PSGRegisters[] */
 	PSGRegisters[PSGRegisterSelect] = val;
@@ -254,7 +277,6 @@ void PSG_Set_DataRegister(Uint8 val)
 	  PSGRegisters[PSGRegisterSelect] &= 0x1f;	/* only keep bits 0 - 4 */
 
 
-
 	if ( PSGRegisterSelect < NUM_PSG_SOUND_REGISTERS )
 	{
 		/* Copy sound related registers 0..13 to the sound module's internal buffer */
@@ -293,20 +315,20 @@ void PSG_Set_DataRegister(Uint8 val)
 		if ( (PSGRegisters[PSG_REG_IO_PORTA]&(1<<1)) == 0 )
 		{
 			/* floppy drive A is ON */
-			Statusbar_SetFloppyLed(DRIVE_LED_A, TRUE);
+			Statusbar_SetFloppyLed(DRIVE_LED_A, true);
 		}
 		else
 		{
-			Statusbar_SetFloppyLed(DRIVE_LED_A, FALSE);
+			Statusbar_SetFloppyLed(DRIVE_LED_A, false);
 		}
 		if ( (PSGRegisters[PSG_REG_IO_PORTA]&(1<<2)) == 0 )
 		{
 			/* floppy drive B is ON */
-			Statusbar_SetFloppyLed(DRIVE_LED_B, TRUE);
+			Statusbar_SetFloppyLed(DRIVE_LED_B, true);
 		}
 		else
 		{
-			Statusbar_SetFloppyLed(DRIVE_LED_B, FALSE);
+			Statusbar_SetFloppyLed(DRIVE_LED_B, false);
 		}
 
 		/* Bit 3 - Centronics as input */
@@ -356,14 +378,14 @@ void PSG_ff8800_ReadByte(void)
 
 	IoMem[IoAccessCurrentAddress] = PSG_Get_DataRegister();
 
-	if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_PSG_READ ) )
-	  {
-	    int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);;
-	    int nLineCycles = nFrameCycles % nCyclesPerLine;
-	    HATARI_TRACE_PRINT ( "ym read data %x=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n" ,
-		IoAccessCurrentAddress , IoMem[IoAccessCurrentAddress] ,
-		nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles );
-	  }
+	if (LOG_TRACE_LEVEL(TRACE_PSG_READ))
+	{
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		LOG_TRACE_PRINT("ym read data %x=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
+		                IoAccessCurrentAddress, IoMem[IoAccessCurrentAddress],
+		                FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
+	}
 }
 
 
@@ -377,14 +399,14 @@ void PSG_ff880x_ReadByte(void)
 
 	IoMem[IoAccessCurrentAddress] = 0xff;
 
-	if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_PSG_READ ) )
-	  {
-	    int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);;
-	    int nLineCycles = nFrameCycles % nCyclesPerLine;
-	    HATARI_TRACE_PRINT ( "ym read void %x=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n" ,
-		IoAccessCurrentAddress , IoMem[IoAccessCurrentAddress] ,
-		nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles );
-	  }
+	if (LOG_TRACE_LEVEL(TRACE_PSG_READ))
+	{
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		LOG_TRACE_PRINT("ym read void %x=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
+		                IoAccessCurrentAddress, IoMem[IoAccessCurrentAddress],
+		                FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
+	}
 }
 
 
@@ -398,14 +420,14 @@ void PSG_ff8800_WriteByte(void)
 //	M68000_WaitState(4);
 	M68000_WaitState(1);				/* [NP] FIXME not 100% accurate, but gives good results */
 
-	if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_PSG_WRITE ) )
-	  {
-	    int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);;
-	    int nLineCycles = nFrameCycles % nCyclesPerLine;
-	    HATARI_TRACE_PRINT ( "ym write %x=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n" ,
-		IoAccessCurrentAddress , IoMem[IoAccessCurrentAddress] ,
-		nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles );
-	  }
+	if (LOG_TRACE_LEVEL(TRACE_PSG_WRITE))
+	{
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		LOG_TRACE_PRINT("ym write %x=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
+		                IoAccessCurrentAddress, IoMem[IoAccessCurrentAddress],
+		                FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
+	}
 
 	PSG_Set_SelectRegister ( IoMem[IoAccessCurrentAddress] );
 }
@@ -424,28 +446,28 @@ void PSG_ff8801_WriteByte(void)
 	{	
 		M68000_WaitState(1);			/* [NP] FIXME not 100% accurate, but gives good results */
 	
-		if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_PSG_WRITE ) )
-		  {
-		    int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);;
-		    int nLineCycles = nFrameCycles % nCyclesPerLine;
-		    HATARI_TRACE_PRINT ( "ym write %x=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n" ,
-			IoAccessCurrentAddress , IoMem[IoAccessCurrentAddress] ,
-			nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles );
-		  }
+		if (LOG_TRACE_LEVEL(TRACE_PSG_WRITE))
+		{
+			int FrameCycles, HblCounterVideo, LineCycles;
+			Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+			LOG_TRACE_PRINT("ym write %x=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
+					IoAccessCurrentAddress, IoMem[IoAccessCurrentAddress],
+					FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
+		}
 	
 		PSG_Set_SelectRegister ( IoMem[IoAccessCurrentAddress] );
 	}
 
 	else
 	{						/* do nothing, just a trace if needed */
-		if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_PSG_WRITE ) )
-		  {
-		    int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);;
-		    int nLineCycles = nFrameCycles % nCyclesPerLine;
-		    HATARI_TRACE_PRINT ( "ym write ignored %x=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n" ,
-			IoAccessCurrentAddress , IoMem[IoAccessCurrentAddress] ,
-			nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles );
-		  }
+		if (LOG_TRACE_LEVEL(TRACE_PSG_WRITE))
+		{
+			int FrameCycles, HblCounterVideo, LineCycles;
+			Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+			LOG_TRACE_PRINT("ym write ignored %x=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
+					IoAccessCurrentAddress, IoMem[IoAccessCurrentAddress],
+					FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
+		}
 	}
 }
 
@@ -459,14 +481,14 @@ void PSG_ff8802_WriteByte(void)
 //	M68000_WaitState(4);
 	M68000_WaitState(1);				/* [NP] FIXME not 100% accurate, but gives good results */
 
-	if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_PSG_WRITE ) )
-	  {
-	    int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);;
-	    int nLineCycles = nFrameCycles % nCyclesPerLine;
-	    HATARI_TRACE_PRINT ( "ym write %x=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n" ,
-		IoAccessCurrentAddress , IoMem[IoAccessCurrentAddress] ,
-		nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles );
-	  }
+	if (LOG_TRACE_LEVEL(TRACE_PSG_WRITE))
+	{
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		LOG_TRACE_PRINT("ym write %x=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
+				IoAccessCurrentAddress, IoMem[IoAccessCurrentAddress],
+				FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
+	}
 
 	PSG_Set_DataRegister ( IoMem[IoAccessCurrentAddress] );
 }
@@ -485,32 +507,27 @@ void PSG_ff8803_WriteByte(void)
 	{	
 		M68000_WaitState(1);			/* [NP] FIXME not 100% accurate, but gives good results */
 	
-		if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_PSG_WRITE ) )
-		  {
-		    int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);;
-		    int nLineCycles = nFrameCycles % nCyclesPerLine;
-		    HATARI_TRACE_PRINT ( "ym write %x=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n" ,
-			IoAccessCurrentAddress , IoMem[IoAccessCurrentAddress] ,
-			nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles );
-		  }
+		if (LOG_TRACE_LEVEL(TRACE_PSG_WRITE))
+		{
+			int FrameCycles, HblCounterVideo, LineCycles;
+			Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+			LOG_TRACE_PRINT("ym write %x=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
+					IoAccessCurrentAddress, IoMem[IoAccessCurrentAddress],
+					FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
+		}
 		
 		PSG_Set_DataRegister ( IoMem[IoAccessCurrentAddress] );
 	}
 
 	else
 	{						/* do nothing, just a trace if needed */
-		if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_PSG_WRITE ) )
-		  {
-		    int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);;
-		    int nLineCycles = nFrameCycles % nCyclesPerLine;
-		    HATARI_TRACE_PRINT ( "ym write ignored %x=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n" ,
-			IoAccessCurrentAddress , IoMem[IoAccessCurrentAddress] ,
-			nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles );
-		  }
+		if (LOG_TRACE_LEVEL(TRACE_PSG_WRITE))
+		{
+			int FrameCycles, HblCounterVideo, LineCycles;
+			Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+			LOG_TRACE_PRINT("ym write ignored %x=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
+					IoAccessCurrentAddress, IoMem[IoAccessCurrentAddress],
+					FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
+		}
 	}
 }
-
-
-
-
-
