@@ -39,9 +39,11 @@ const char Statusbar_fileid[] = "Hatari statusbar.c : " __DATE__ " " __TIME__;
 #include "sdlgui.h"
 #include "statusbar.h"
 #include "tos.h"
+#include "screen.h"
 #include "video.h"
 #include "wavFormat.h"
 #include "ymFormat.h"
+#include "avi_record.h"
 
 #define DEBUG 0
 #if DEBUG
@@ -83,7 +85,7 @@ static Uint32 LedColorOn, LedColorOff, RecColorOn, RecColorOff;
 static Uint32 GrayBg, LedColorBg;
 
 
-#define MAX_MESSAGE_LEN 23
+#define MAX_MESSAGE_LEN 33
 typedef struct msg_item {
 	struct msg_item *next;
 	char msg[MAX_MESSAGE_LEN+1];
@@ -96,8 +98,10 @@ static msg_item_t DefaultMessage;
 static msg_item_t *MessageList = &DefaultMessage;
 static SDL_Rect MessageRect;
 
+/* rect for both frame skip value and fast forward indicator */
 static SDL_Rect FrameSkipsRect;
 static int nOldFrameSkips;
+static int bOldFastForward;
 
 
 /* screen height above statusbar and height of statusbar below screen */
@@ -105,6 +109,25 @@ static int ScreenHeight;
 static int StatusbarHeight;
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Return statusbar height for given width and height
+ */
+int Statusbar_GetHeightForSize(int width, int height)
+{
+	if (ConfigureParams.Screen.bShowStatusbar) {
+		/* Should check the same thing as SDLGui_SetScreen()
+		 * does to decide the font size.
+		 */
+		if (width >= 640 && height >= (400-24)) {
+			return 24;
+		} else {
+			return 12;
+		}
+	}
+	return 0;
+}
+
 /*-----------------------------------------------------------------------*/
 /**
  * Set screen height used for statusbar height calculation.
@@ -116,18 +139,8 @@ static int StatusbarHeight;
 int Statusbar_SetHeight(int width, int height)
 {
 	ScreenHeight = height;
-	if (ConfigureParams.Screen.bShowStatusbar) {
-		/* Should check the same thing as SDLGui_SetScreen()
-		 * does to decide the font size.
-		 */
-		if (width >= 640 && height >= (400-24)) {
-			StatusbarHeight = 24;
-		} else {
-			StatusbarHeight = 12;
-		}
-	} else {
-		StatusbarHeight = 0;
-	}
+	StatusbarHeight = Statusbar_GetHeightForSize(width, height);
+	DEBUGPRINT(("Statusbar_SetHeight(%d, %d) -> %d\n", width, height, StatusbarHeight));
 	return StatusbarHeight;
 }
 
@@ -202,6 +215,7 @@ void Statusbar_Init(SDL_Surface *surf)
 	int i, fontw, fonth, offset;
 	const char *text[MAX_DRIVE_LEDS] = { "A:", "B:", "HD:" };
 
+	DEBUGPRINT(("Statusbar_Init()\n"));
 	assert(surf);
 
 	/* dark green and light green for leds themselves */
@@ -224,6 +238,7 @@ void Statusbar_Init(SDL_Surface *surf)
 		StatusbarHeight = 0;
 	}
 	if (!StatusbarHeight) {
+		DEBUGPRINT(("Doesn't fit <- Statusbar_Init()\n"));
 		return;
 	}
 
@@ -249,10 +264,10 @@ void Statusbar_Init(SDL_Surface *surf)
 	SDL_FillRect(surf, &sbarbox, GrayBg);
 
 	/* led size */
-	LedRect.w = 2*fonth;
+	LedRect.w = fonth/2;
 	LedRect.h = fonth - 4;
 	LedRect.y = ScreenHeight + StatusbarHeight/2 - LedRect.h/2;
-	
+
 	/* black box for the leds */
 	ledbox = LedRect;
 	ledbox.y -= 1;
@@ -274,7 +289,7 @@ void Statusbar_Init(SDL_Surface *surf)
 		SDL_FillRect(surf, &LedRect, LedColorOff);
 
 		Led[i].offset = offset;
-		offset += LedRect.w + 2*fontw;
+		offset += LedRect.w + fontw;
 	}
 
 	/* draw frameskip */
@@ -282,13 +297,19 @@ void Statusbar_Init(SDL_Surface *surf)
 	FrameSkipsRect.y = MessageRect.y;
 	SDLGui_Text(FrameSkipsRect.x, FrameSkipsRect.y, "FS:");
 	FrameSkipsRect.x += 3 * fontw + fontw/2;
-	FrameSkipsRect.w = fontw;
+	FrameSkipsRect.w = 4 * fontw;
 	FrameSkipsRect.h = fonth;
-	SDLGui_Text(FrameSkipsRect.x, FrameSkipsRect.y, "0");
+
+	if(ConfigureParams.System.bFastForward) {
+		SDLGui_Text(FrameSkipsRect.x, FrameSkipsRect.y, "0 >>");
+	} else {
+		SDLGui_Text(FrameSkipsRect.x, FrameSkipsRect.y, "0");
+	}
+
 	nOldFrameSkips = 0;
-	
+
 	/* intialize messages */
-	MessageRect.x = FrameSkipsRect.x + 2 * fontw;
+	MessageRect.x = FrameSkipsRect.x + FrameSkipsRect.w + fontw;
 	MessageRect.w = MAX_MESSAGE_LEN * fontw;
 	MessageRect.h = fonth;
 	for (item = MessageList; item; item = item->next) {
@@ -306,7 +327,7 @@ void Statusbar_Init(SDL_Surface *surf)
 
 	/* and blit statusbar on screen */
 	SDL_UpdateRects(surf, 1, &sbarbox);
-	DEBUGPRINT(("Draw statusbar\n"));
+	DEBUGPRINT(("Drawn <- Statusbar_Init()\n"));
 }
 
 
@@ -361,6 +382,21 @@ void Statusbar_UpdateInfo(void)
 {
 	char *end = DefaultMessage.msg;
 
+	/* CPU MHz */
+	if (ConfigureParams.System.nCpuFreq > 9) {
+		*end++ = '0' + ConfigureParams.System.nCpuFreq / 10;
+	}
+	*end++ = '0' + ConfigureParams.System.nCpuFreq % 10;
+	end = Statusbar_AddString(end, "MHz/");
+
+	/* CPU type */
+	if(ConfigureParams.System.nCpuLevel > 0) {
+		*end++ = '0';
+		*end++ = '0' + ConfigureParams.System.nCpuLevel % 10;
+		*end++ = '0';
+		*end++ = '/';
+	}
+
 	/* amount of memory */
 	if (ConfigureParams.Memory.nMemorySize > 9) {
 		*end++ = '1';
@@ -373,7 +409,7 @@ void Statusbar_UpdateInfo(void)
 		}
 	}
 	end = Statusbar_AddString(end, "MB ");
-	
+
 	/* machine type */
 	switch (ConfigureParams.System.nMachineType) {
 	case MACHINE_ST:
@@ -382,6 +418,9 @@ void Statusbar_UpdateInfo(void)
 	case MACHINE_STE:
 		end = Statusbar_AddString(end, "STE");
 		break;
+	case MACHINE_MEGA_STE:
+		end = Statusbar_AddString(end, "Mega STE");
+		break;
 	case MACHINE_TT:
 		end = Statusbar_AddString(end, "TT");
 		break;
@@ -404,8 +443,10 @@ void Statusbar_UpdateInfo(void)
 		*end++ = '0' + (TosVersion & 0xf);
 	}
 	*end = '\0';
+
 	assert(end - DefaultMessage.msg < MAX_MESSAGE_LEN);
 	DEBUGPRINT(("Set default message: '%s'\n", DefaultMessage.msg));
+	/* make sure default message gets (re-)drawn when next checked */
 	DefaultMessage.shown = false;
 }
 
@@ -437,7 +478,7 @@ static void Statusbar_ShowMessage(SDL_Surface *surf, Uint32 ticks)
 
 	if (MessageList->shown) {
 		if (!MessageList->expire) {
-			/* last/default message */
+			/* last/default message newer expires */
 			return;
 		}
 		if (MessageList->expire > ticks) {
@@ -540,7 +581,7 @@ static void Statusbar_OverlayDraw(SDL_Surface *surf)
 	int i;
 
 	assert(surf);
-	if (bRecordingYM || bRecordingWav || bRecordingAnimation) {
+	if (bRecordingYM || bRecordingWav || bRecordingAvi) {
 		Statusbar_OverlayDrawLed(surf, RecColorOn);
 	}
 	for (i = 0; i < MAX_DRIVE_LEDS; i++) {
@@ -591,6 +632,11 @@ void Statusbar_Update(SDL_Surface *surf)
 	}
 	assert(surf);
 	/* Statusbar_Init() not called before this? */
+#if DEBUG
+	if (surf->h != ScreenHeight + StatusbarHeight) {
+		printf("%d != %d + %d\n", surf->h, ScreenHeight, StatusbarHeight);
+	}
+#endif
 	assert(surf->h == ScreenHeight + StatusbarHeight);
 
 	rect = LedRect;
@@ -613,22 +659,35 @@ void Statusbar_Update(SDL_Surface *surf)
 		SDL_UpdateRects(surf, 1, &rect);
 		DEBUGPRINT(("LED[%d] = %s\n", i, Led[i].state?"ON":"OFF"));
 	}
+
 	Statusbar_ShowMessage(surf, currentticks);
 
-	if (nFrameSkips != nOldFrameSkips) {
-		char fscount[2] = { '\0', '\0' };
+	if (nOldFrameSkips != nFrameSkips ||
+	    bOldFastForward != ConfigureParams.System.bFastForward) {
+		char fscount[5];
+		int end = 2;
+
 		if (nFrameSkips < 10)
 			fscount[0] = '0' + nFrameSkips;
 		else
 			fscount[0] = 'X';
+		fscount[1] = ' ';
+		if(ConfigureParams.System.bFastForward) {
+			fscount[2] = '>';
+			fscount[3] = '>';
+			end = 4;
+		}
+		fscount[end] = '\0';
+
 		SDL_FillRect(surf, &FrameSkipsRect, GrayBg);
 		SDLGui_Text(FrameSkipsRect.x, FrameSkipsRect.y, fscount);
 		SDL_UpdateRects(surf, 1, &FrameSkipsRect);
 		DEBUGPRINT(("FS = %s\n", fscount));
 		nOldFrameSkips = nFrameSkips;
+		bOldFastForward = ConfigureParams.System.bFastForward;
 	}
 
-	if ((bRecordingYM || bRecordingWav || bRecordingAnimation)
+	if ((bRecordingYM || bRecordingWav || bRecordingAvi)
 	    != bOldRecording) {
 		bOldRecording = !bOldRecording;
 		if (bOldRecording) {
