@@ -23,7 +23,7 @@ const char XBios_fileid[] = "Hatari xbios.c : " __DATE__ " " __TIME__;
 #include "xbios.h"
 
 
-#define XBIOS_DEBUG 0	/* for floppy read/write */
+#define XBIOS_DEBUG 0	/* for floppy read/write & rsconf */
 
 
 /* List of Atari ST RS-232 baud rates */
@@ -55,19 +55,19 @@ static const int BaudRates[] =
 static bool XBios_Floprd(Uint32 Params)
 {
 #if XBIOS_DEBUG
-	char *pBuffer;
+	Uint32 pBuffer;
 	Uint16 Dev,Sector,Side,Track,Count;
 
 	/* Read details from stack */
-	pBuffer = (char *)STRAM_ADDR(STMemory_ReadLong(Params+SIZE_WORD));
+	pBuffer = STMemory_ReadLong(Params+SIZE_WORD);
 	Dev = STMemory_ReadWord(Params+SIZE_WORD+SIZE_LONG+SIZE_LONG);
 	Sector = STMemory_ReadWord(Params+SIZE_WORD+SIZE_LONG+SIZE_LONG+SIZE_WORD);
 	Track = STMemory_ReadWord(Params+SIZE_WORD+SIZE_LONG+SIZE_LONG+SIZE_WORD+SIZE_WORD);
 	Side = STMemory_ReadWord(Params+SIZE_WORD+SIZE_LONG+SIZE_LONG+SIZE_WORD+SIZE_WORD+SIZE_WORD);
 	Count = STMemory_ReadWord(Params+SIZE_WORD+SIZE_LONG+SIZE_LONG+SIZE_WORD+SIZE_WORD+SIZE_WORD+SIZE_WORD);
 
-	Log_Printf(LOG_DEBUG, "FLOPRD %s,%d,%d,%d,%d at addr 0x%X\n", EmulationDrives[Dev].szFileName,
-	           Side, Track, Sector, Count, M68000_GetPC());
+	Log_Printf(LOG_DEBUG, "FLOPRD %s,0x%x,%d,%d,%d,%d at addr 0x%X\n", EmulationDrives[Dev].sFileName,
+	           pBuffer, Side, Track, Sector, Count, M68000_GetPC());
 #endif
 
 	return false;
@@ -81,19 +81,19 @@ static bool XBios_Floprd(Uint32 Params)
 static bool XBios_Flopwr(Uint32 Params)
 {
 #if XBIOS_DEBUG
-	char *pBuffer;
+	Uint32 pBuffer;
 	Uint16 Dev,Sector,Side,Track,Count;
 
 	/* Read details from stack */
-	pBuffer = (char *)STRAM_ADDR(STMemory_ReadLong(Params+SIZE_WORD));
+	pBuffer = STMemory_ReadLong(Params+SIZE_WORD);
 	Dev = STMemory_ReadWord(Params+SIZE_WORD+SIZE_LONG+SIZE_LONG);
 	Sector = STMemory_ReadWord(Params+SIZE_WORD+SIZE_LONG+SIZE_LONG+SIZE_WORD);
 	Track = STMemory_ReadWord(Params+SIZE_WORD+SIZE_LONG+SIZE_LONG+SIZE_WORD+SIZE_WORD);
 	Side = STMemory_ReadWord(Params+SIZE_WORD+SIZE_LONG+SIZE_LONG+SIZE_WORD+SIZE_WORD+SIZE_WORD);
 	Count = STMemory_ReadWord(Params+SIZE_WORD+SIZE_LONG+SIZE_LONG+SIZE_WORD+SIZE_WORD+SIZE_WORD+SIZE_WORD);
 
-	Log_Printf(LOG_DEBUG, "FLOPWR %s,%d,%d,%d,%d at addr 0x%X\n", EmulationDrives[Dev].szFileName,
-	           Side, Track, Sector, Count, M68000_GetPC());
+	Log_Printf(LOG_DEBUG, "FLOPWR %s,0x%x,%d,%d,%d,%d at addr 0x%X\n", EmulationDrives[Dev].sFileName,
+	           pBuffer, Side, Track, Sector, Count, M68000_GetPC());
 #endif
 
 	return false;
@@ -106,19 +106,23 @@ static bool XBios_Flopwr(Uint32 Params)
  */
 static bool XBios_Rsconf(Uint32 Params)
 {
-	short int Baud,Ctrl,Ucr,Rsr,Tsr,Scr;
+	Sint16 Baud,Ctrl,Ucr;
 
 	Baud = STMemory_ReadWord(Params+SIZE_WORD);
 	Ctrl = STMemory_ReadWord(Params+SIZE_WORD+SIZE_WORD);
 	Ucr = STMemory_ReadWord(Params+SIZE_WORD+SIZE_WORD+SIZE_WORD);
+#if XBIOS_DEBUG
+	Sint16 Rsr,Tsr,Scr;
 	Rsr = STMemory_ReadWord(Params+SIZE_WORD+SIZE_WORD+SIZE_WORD+SIZE_WORD);
 	Tsr = STMemory_ReadWord(Params+SIZE_WORD+SIZE_WORD+SIZE_WORD+SIZE_WORD+SIZE_WORD);
 	Scr = STMemory_ReadWord(Params+SIZE_WORD+SIZE_WORD+SIZE_WORD+SIZE_WORD+SIZE_WORD+SIZE_WORD);
-
+	Log_Printf(LOG_DEBUG, "RSCONF baud,ctrl,ucr,rsr,tsr,scr: %d,%d,%d,%d,%d,%d\n",
+		   Baud, Ctrl, Ucr, Rsr, Tsr, Scr);
+#endif
 	/* Set baud rate and other configuration, if RS232 emaulation is enabled */
 	if (ConfigureParams.RS232.bEnableRS232)
 	{
-		if (Baud>=0 && Baud<=15)
+		if (Baud >= 0 && Baud < ARRAYSIZE(BaudRates))
 		{
 			/* Convert ST baud rate index to value */
 			int BaudRate = BaudRates[Baud];
@@ -173,6 +177,191 @@ static bool XBios_HatariControl(Uint32 Params)
 }
 
 
+#if ENABLE_TRACING
+/**
+ * Map XBIOS call opcode to XBIOS function name
+ */
+static const char* XBios_Call2Name(Uint16 opcode)
+{
+	static const char* names[] = {
+		"Initmous",
+		"Ssbrk",
+		"Physbase",
+		"Logbase",
+		"Getrez",
+		"Setscreen",
+		"Setpalette",
+		"Setcolor",
+		"Floprd",
+		"Flopwr",
+		"Flopfmt",
+		"Dbmsg",
+		"Midiws",
+		"Mfpint",
+		"Iorec",
+		"Rsconf",
+		"Keytbl",
+		"Random",
+		"Protobt",
+		"Flopver",
+		"Scrdmp",
+		"Cursconf",
+		"Settime",
+		"Gettime",
+		"Bioskeys",
+		"Ikbdws",
+		"Jdisint",
+		"Jenabint",
+		"Giaccess",
+		"Offgibit",
+		"Ongibit",
+		"Xbtimer",
+		"Dosound",
+		"Setprt",
+		"Kbdvbase",
+		"Kbrate",
+		"Prtblk",
+		"Vsync",
+		"Supexec",
+		"Puntaes",
+		NULL,	/* 40 */
+		"Floprate",
+		"DMAread",
+		"DMAwrite",
+		"Bconmap",
+		NULL,	/* 45 */
+		"NVMaccess",
+		NULL,	/* 47 */
+		"Metainit",
+		NULL,	/* 49 */
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,	/* 63 */
+		"Blitmode",
+		NULL,	/* 65 */
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,	/* 79 */
+		"EsetShift",
+		"EgetShift",
+		"EsetBank",
+		"EsetColor",
+		"EsetPalette",
+		"EgetPalette",
+		"EsetGray",
+		"EsetSmear",
+		"VsetMode",
+		"VgetMonitor",
+		"VsetSync",
+		"VgetSize",
+		NULL,	/* 92 */
+		"VsetRGB",
+		"VgetRGB",
+		NULL,	/* 95 */
+		"Dsp_DoBlock",
+		"Dsp_BlkHandShake",
+		"Dsp_BlkUnpacked",
+		"Dsp_InStream",
+		"Dsp_OutStream",
+		"Dsp_IOStream",
+		"Dsp_RemoveInterrupts",
+		"Dsp_GetWordSize",
+		"Dsp_Lock",
+		"Dsp_Unlock",
+		"Dsp_Available",
+		"Dsp_Reserve",
+		"Dsp_LoadProg",
+		"Dsp_ExecProg",
+		"Dsp_ExecBoot",
+		"Dsp_LodToBinary",
+		"Dsp_TriggerHC",
+		"Dsp_RequestUniqueAbility",
+		"Dsp_GetProgAbility",
+		"Dsp_FlushSubroutines",
+		"Dsp_LoadSubroutine",
+		"Dsp_InqSubrAbility",
+		"Dsp_RunSubroutine",
+		"Dsp_Hf0",
+		"Dsp_Hf1",
+		"Dsp_Hf2",
+		"Dsp_Hf3",
+		"Dsp_BlkWords",
+		"Dsp_BlkBytes",
+		"Dsp_HStat",
+		"Dsp_SetVectors",
+		"Dsp_MultBlocks",
+		"Locksnd",
+		"Unlocksnd",
+		"Soundcmd",
+		"Setbuffer",
+		"Setmode",
+		"Settracks",
+		"Setmontracks",
+		"Setinterrupt",
+		"Buffoper",
+		"Dsptristate",
+		"Gpio",
+		"Devconnect",
+		"Sndstatus",
+		"Buffptr",
+		NULL,	/* 142 */
+		NULL,
+		NULL,
+		NULL,
+		"VsetMask",
+		NULL,	/* 147 */
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,
+		NULL,	/* 164 */
+		"WavePlay"
+	};
+	if (opcode < ARRAYSIZE(names) && names[opcode]) {
+		return names[opcode];
+	}
+	if (opcode == 255) {
+		return "HatariControl";
+	}
+	return "???";
+}
+#endif
+
+
 /**
  * Check if we need to re-direct XBios call to our own routines
  */
@@ -185,26 +374,23 @@ bool XBios(void)
 	Params = Regs[REG_A7];
 	XBiosCall = STMemory_ReadWord(Params);
 
+	LOG_TRACE(TRACE_OS_XBIOS, "XBIOS %hd (%s)\n",
+		  XBiosCall, XBios_Call2Name(XBiosCall));
+
 	switch (XBiosCall)
 	{
 	 case 8:
-		LOG_TRACE(TRACE_OS_XBIOS, "XBIOS Floprd()\n");
 		return XBios_Floprd(Params);
 	 case 9:
-		LOG_TRACE(TRACE_OS_XBIOS, "XBIOS Flopwr()\n");
 		return XBios_Flopwr(Params);
 	 case 15:
-		LOG_TRACE(TRACE_OS_XBIOS, "XBIOS Rsconf()\n");
 		return XBios_Rsconf(Params);
 	 case 20:
-		LOG_TRACE(TRACE_OS_XBIOS, "XBIOS Scrdmp()\n");
 		return XBios_Scrdmp(Params);
 	 case 255:
-		LOG_TRACE(TRACE_OS_XBIOS, "XBIOS HatariControl()\n");
 		return XBios_HatariControl(Params);
 
 	 default:  /* Call as normal! */
-		LOG_TRACE(TRACE_OS_XBIOS, "XBIOS %d\n", XBiosCall);
 		return false;
 	}
 }
